package org.implab.gradle.containers.tasks;

import java.io.File;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.gradle.api.DefaultTask;
import org.gradle.api.file.Directory;
import org.gradle.api.file.DirectoryProperty;
import org.gradle.api.logging.Logger;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.containers.ContainerExtension;
import org.implab.gradle.containers.ExecuteMixin;
import org.implab.gradle.containers.PropertiesMixin;
import org.implab.gradle.containers.cli.DockerTraits;

public abstract class DockerCliTask extends DefaultTask implements PropertiesMixin, ExecuteMixin {

    private final Property<String> cliCmd;

    private final DirectoryProperty workingDir;

    public DockerCliTask() {
        cliCmd = property(String.class)
                .convention(getContainerExtension().getCliCmd());

        workingDir = directoryProperty();
    }

    @Internal
    protected Optional<String> getSubCommand() {
        return Optional.empty();
    }

    @Internal
    protected Collection<String> getSubCommandArguments() {
        return Collections.emptyList();
    }

    @Input
    public Property<String> getCliCmd() {
        return cliCmd;
    }

    public void setCliCmd(String cliCmd) {
        this.cliCmd.set(cliCmd);
    }

    @Internal
    protected ContainerExtension getContainerExtension() {
        return getProject().getExtensions().getByType(ContainerExtension.class);
    }

    @TaskAction
    public void Run() throws Exception {
        Execute();
    }

    public void preparingCommand(List<String> commandLine) {
        commandLine.add(cliCmd.get());
        getSubCommand().ifPresent(commandLine::add);
        getSubCommandArguments().forEach(commandLine::add);
    }

    @Override
    @Internal
    public Optional<File> getWorkingDir() {
        return workingDir
            .map(Directory::getAsFile)
            .map(Optional::of)
            .getOrElse(Optional.empty());
    }

    protected void setWorkingDir(Provider<File> workingDir) {
        this.workingDir.fileProvider(workingDir);
    }

    protected void setWorkingDir(File workingDir) {
        this.workingDir.set(workingDir);
    }

    protected DockerTraits docker() {
        return new TaskDockerTraits();
    }

    class TaskDockerTraits extends DockerTraits {

        @Override
        public Logger getLogger() {
            return DockerCliTask.this.getLogger();
        }

        @Override
        public Optional<File> getWorkingDir() {
            return DockerCliTask.this.getWorkingDir();
        }

        @Override
        public String getCliCmd() {
            return cliCmd.get();
        }

    }
}
